#include <assert.h>
#include <conio.h>
#include <dos.h>
#include <mem.h>
#include <stdio.h>
#include <string.h>

typedef unsigned char byte;
typedef signed char schar;
typedef char bool;

const int MaxX = 319; const int MaxY = 199;
const byte * VADDR = (byte *) 0xA0000000LU;

void init_gr_mode (int mode)
{
 _AL = mode; _AH = 0x00; geninterrupt (0x10);
}

void pset (int x, int y, byte c)
{
 * ((byte *) VADDR + y * 320 + x) = c;
}

void set_palette (byte * table, int st, int sz)
{
 _ES = FP_SEG(table);
 _DX = FP_OFF(table);
 _BX = st;
 _CX = sz;
 _AH = 0x10;
 _AL = 0x12;
 geninterrupt(0x10);
}

const byte XMax = 100, YMax = 100, MaxAnts = 200;
const int MaxState = 10000;
typedef byte dir;
typedef byte lr_type;
typedef byte sense_dir;
typedef byte colour;
typedef byte condition;
typedef int state;
typedef byte instruction;
struct pos {int x, y;};
struct ant {colour c; pos p; int state; int resting; dir d; bool has_food;};
enum Edir {DirE, DirSSE, DirSSW, DirW, DirNNW, DirNNE};
enum Ecolour {Red = 0, Black = 1};
enum Elr_type {Left = 5, Right = 1};
enum Esense_dir {Here = 0, Ahead = 1, LeftAhead = 2, RightAhead = 3};
enum Efield {Empty = 0, Rock = 1, RedAnthill = 2, BlackAnthill = 3};
enum Econdition {Friend = 6, Foe, FriendWithFood, FoeWithFood, Food, Rocky,
 FoeMarker, Home, FoeHome};
enum Einstruction {Sense, Mark, Unmark, PickUp, Drop, Turn, Move, Flip};

int xsz, ysz;
byte far field [XMax][YMax];
int far food [XMax][YMax];
byte far ids [XMax][YMax];
byte far marks [2][XMax][YMax];
ant far ants [MaxAnts];
int an;
unsigned long randomseed;
instruction far instr[2][MaxState];
state far state1[2][MaxState];
state far state2[2][MaxState];
int far arg[2][MaxState];

void adjacent_cell (pos & ans, pos p, dir d)
{
 switch (d)
 {
  case DirE: ans.x = p.x + 1; ans.y = p.y; break;
  case DirSSE: ans.x = p.x + (p.y & 1); ans.y = p.y + 1; break;
  case DirSSW: ans.x = p.x + (p.y & 1) - 1; ans.y = p.y + 1; break;
  case DirW: ans.x = p.x - 1; ans.y = p.y; break;
  case DirNNW: ans.x = p.x + (p.y & 1) - 1; ans.y = p.y - 1; break;
  case DirNNE: ans.x = p.x + (p.y & 1); ans.y = p.y - 1; break;
  default: assert (0); break;
 }
}

dir turn (lr_type lr, dir d)
{
 d += lr;
 if (d >= 6) d -= 6;
 return (d);
}

void sensed_cell (pos & ans, pos p, dir d, sense_dir sd)
{
 switch (sd)
 {
  case Here: ans.x = p.x; ans.y = p.y; break;
  case Ahead: adjacent_cell (ans, p, d); break;
  case LeftAhead: adjacent_cell (ans, p, turn (d, Left)); break;
  case RightAhead: adjacent_cell (ans, p, turn (d, Right)); break;
  default: assert (0); break;
 }
}

colour other_colour (colour c)
{
 return (c ^ 1);
}

int get_state (int id) {return (ants[id].state);}
int get_colour (int id) {return (ants[id].c);}
int get_resting (int id) {return (ants[id].resting);}
int get_direction (int id) {return (ants[id].d);}
bool get_has_food (int id) {return (ants[id].has_food);}

void set_state (int id, int new_state) {ants[id].state = new_state;};
void set_resting (int id, int new_resting) {ants[id].resting = new_resting;};
void set_direction (int id, int new_d) {ants[id].d = new_d;};
void set_has_food (int id, int new_has_food)
{ants[id].has_food = new_has_food;};

bool some_ant_is_at (pos p) {return (ids[p.x][p.y] != MaxAnts);}
int get_ant_at (pos p) {return (ids[p.x][p.y]);}

void set_ant_at (pos p, int id) {ids[p.x][p.y] = id;}
void clear_ant_at (pos p) {ids[p.x][p.y] = MaxAnts;}

bool ant_is_alive (int id) {return (ants[id].state < MaxState);}
void find_ant (pos & ans, int id)
{ans.x = ants[id].p.x; ans.y = ants[id].p.y;}
void kill_ant_at (pos p)
{ants[ids[p.x][p.y]].state = MaxState; clear_ant_at (p);}

int get_food_at (pos p) {return (food[p.x][p.y]);}
void set_food_at (pos p, int new_food) {food[p.x][p.y] = new_food;}
bool anthill_at (pos p, colour c) {return (field[p.x][p.y] == c + 2);}

void set_marker_at (pos p, colour c, byte n)
{marks[c][p.x][p.y] = marks[c][p.x][p.y] | (1 << n);}
void clear_marker_at (pos p, colour c, byte n)
{marks[c][p.x][p.y] = marks[c][p.x][p.y] & (0xFF - (1 << n));}
bool check_marker_at (pos p, colour c, byte n)
{return ((marks[c][p.x][p.y] & (1 << n)) > 0);}
bool check_any_marker_at (pos p, colour c)
{return (marks[c][p.x][p.y] > 0);}

bool cell_matches (pos p, condition cond, colour c)
{
 if (field[p.x][p.y] == Rock) return (cond == Rocky);
 switch (cond)
 {
  case Friend: return (some_ant_is_at (p) &&
   (get_colour (get_ant_at (p)) == c));
  case Foe: return (some_ant_is_at (p) &&
   (get_colour (get_ant_at (p)) != c));
  case FriendWithFood: return (some_ant_is_at (p) &&
   (get_colour (get_ant_at (p)) == c) && get_has_food (get_ant_at (p)));
  case FoeWithFood: return (some_ant_is_at (p) &&
   (get_colour (get_ant_at (p)) != c) && get_has_food (get_ant_at (p)));
  case Food: return (get_food_at (p) > 0);
  case Rocky: return (0);
  case FoeMarker: return (check_any_marker_at (p, other_colour (c)));
  case Home: return (anthill_at (p, c));
  case FoeHome: return (anthill_at (p, other_colour (c)));
  default: if (cond <= 5) return (check_marker_at (p, c, cond));
   else assert (0);
 }
 return (0);
}

byte adjacent_ants (pos p, colour c)
{
 byte n = 0;
 pos cur;
 for (dir d = 0; d < 6; d++)
 {
  adjacent_cell (cur, p, d);
  if (some_ant_is_at (cur) && (get_colour (get_ant_at (cur)) == c)) n++;
 }
 return (n);
}

void check_for_surrounded_ant_at (pos p)
{
 if (some_ant_is_at (p))
 {
  int a = get_ant_at (p);
  if (adjacent_ants (p, other_colour (get_colour (a))) >= 5)
  {
   set_food_at (p, get_food_at (p) + 3 + get_has_food (a));
   kill_ant_at (p);
  }
 }
}

void check_for_surrounded_ants (pos p)
{
 pos cur;
 check_for_surrounded_ant_at (p);
 for (dir d = 0; d < 6; d++)
 {
  adjacent_cell (cur, p, d);
  check_for_surrounded_ant_at (cur);
 }
}

void next_random () {randomseed = randomseed * 22695477LU + 1;}

void set_randomseed (unsigned long n)
{
 randomseed = n;
 for (int i = 0; i < 3; i++)
  next_random ();
}

int randomint (int n)
{
 next_random ();
 return ((int) ((randomseed >> 16) & 0x3FFF) % n);
}

void step (int id)
{
 pos p, cur;
 if (ant_is_alive (id))
 {
  find_ant (p, id);
  if (get_resting (id) > 0)
  {
   set_resting (id, get_resting (id) - 1);
  }
  else
  {
   state st = get_state (id);
   colour c = get_colour (id);
   int st1 = state1[c][st], st2 = state2[c][st];
   int k = arg[c][st];
   switch (instr[c][st])
   {
    case Sense:
     sensed_cell (cur, p, get_direction (id), k >> 8);
     set_state (id, (cell_matches (cur, k & 0xFF, c)) ? st1 : st2);
     break;
    case Mark:
     set_marker_at (p, c, k);
     set_state (id, st1);
     break;
    case Unmark:
     clear_marker_at (p, c, k);
     set_state (id, st1);
     break;
    case PickUp:
     if (get_has_food (id) || (get_food_at (p) == 0))
      set_state (id, st2);
     else
     {
      set_food_at (p, get_food_at (p) - 1);
      set_has_food (id, 1);
      set_state (id, st1);
     }
     break;
    case Drop:
     if (get_has_food (id))
     {
      set_food_at (p, get_food_at (p) + 1);
      set_has_food (id, 0);
     }
     set_state (id, st1);
     break;
    case Turn:
     set_direction (id, turn (k, get_direction (id)));
     set_state (id, st1);
     break;
    case Move:
     adjacent_cell (cur, p, get_direction (id));
     if ((field[cur.x][cur.y] == Rock) || (some_ant_is_at (cur)))
      set_state (id, st2);
     else
     {
      clear_ant_at (p);
      set_ant_at (cur, id);
      set_state (id, st1);
      set_resting (id, 14);
      check_for_surrounded_ants (cur);
      ants[id].p.x = cur.x;
      ants[id].p.y = cur.y;
     }
     break;
    case Flip:
     if (randomint (k) == 0)
      set_state (id, st1);
     else
      set_state (id, st2);
     break;
    default: assert (0);
   }
  }
 }
}

const MaxC = 18, MaxFN = 6, MaxBufLen = 256;
char fn[MaxFN][90] = {"sim.cfg", "world.cur", "red.ant", "black.ant",
 "dump.txt", "result.txt"};
unsigned long new_rnd = 12345LU;
unsigned long num_turns = 100000LU;
int colours[MaxC][3];
int to_draw = 0, to_dump = 0;
byte palette[256][3];
byte xcell = 2, ycell = 2;
int delay1 = 0;
unsigned long draw_step, dump_step;

int readstr (FILE * f1, char * str)
{
 int c, n = 0;
 while (((c = getc (f1)) != 0x0A) && (c != 0x0D) && (c != EOF))
 {
  str[n++] = c;
  assert (n < MaxBufLen);
 }
 str[n] = 0;
 if (c == 0x0D) c = getc (f1); // to read 0x0A
 return (c);
}

void readterm (char * ans, int & pos, char * str)
{
 for (int k = 0; (ans[k] = str[pos]) > 0x20; k++, pos++) {}
 ans[k] = 0;
 if (str[pos] == 0x20) pos++;
}

int readnum (char * str)
{
 for (int n = 0, i = 0; (str[i] >= '0') && (str[i] <= '9'); i++)
 {
  n = n * 10 + str[i] - 48;
 }
 assert (i > 0);
 assert (str[i] == 0);
 return (n);
}

void get_config ()
{
 FILE * f1 = fopen (fn[0], "rt");
 if (f1 != NULL)
 {
  fscanf (f1, "%lu", &new_rnd);
  fscanf (f1, "%lu", &num_turns);
  fscanf (f1, "%s", fn[1]);
  fscanf (f1, "%s", fn[2]);
  fscanf (f1, "%s", fn[3]);
  fscanf (f1, "%s", fn[4]);
  fscanf (f1, "%s", fn[5]);
  fscanf (f1, "%d", &to_draw);
  fscanf (f1, "%d", &to_dump);
  fscanf (f1, "%d", &delay1);
  fscanf (f1, "%d", &draw_step);
  fscanf (f1, "%d", &dump_step);
  int i, j, k, l;
  fscanf (f1, "%d", &i);
  xcell = i;
  fscanf (f1, "%d", &j);
  ycell = j;
  for (i = 0; i < MaxC; i++)
  {
   fscanf (f1, "%d %d %d", &colours[i][0], &colours[i][1], &colours[i][2]);
  }
  for (i = 0; i < 6; i++)
   for (j = 0; j < 6; j++)
    for (k = 0; k < 6; k++)
    {
     palette[i * 36 + j * 6 + k][0] = (i * 63) / 5;
     palette[i * 36 + j * 6 + k][1] = (j * 63) / 5;
     palette[i * 36 + j * 6 + k][2] = (k * 63) / 5;
    }
  for (i = 0; i < 40; i++)
  {
   fscanf (f1, "%d %d %d", &j, &k, &l);
   palette[i + 216][0] = j;
   palette[i + 216][1] = k;
   palette[i + 216][2] = l;
  }
  fclose (f1);
 }
}

void get_board ()
{
 FILE * f1 = fopen (fn[1], "rt");
 int c, i, j;
 fscanf (f1, "%d %d", &i, &j);
 xsz = i; ysz = j;
 while (xcell * xsz > MaxX + 1) xcell--;
 while (ycell * ysz > MaxY + 1) ycell--;
 _fmemset (field, 0, sizeof (field));
 _fmemset (food, 0, sizeof (food));
 _fmemset (ids, MaxAnts, sizeof (ids));
 _fmemset (marks, 0, sizeof (marks));
 _fmemset (ants, 0, sizeof (ants));
 for (j = 0; j < ysz; j++)
  for (i = 0; i < xsz; i++)
  {
   while (((c = getc (f1)) <= 0x20) && (c != EOF)) {}
   switch (c)
   {
    case '.': field[i][j] = Empty; break;
    case '#': field[i][j] = Rock; break;
    case '+': field[i][j] = RedAnthill; ids[i][j] = an;
     ants[an].c = Red; ants[an].p.x = i; ants[an].p.y = j;
     ants[an].state = 0; ants[an].resting = 0;
     ants[an].d = DirE; ants[an].has_food = 0;
     an++;
     break;
    case '-': field[i][j] = BlackAnthill; ids[i][j] = an;
     ants[an].c = Black; ants[an].p.x = i; ants[an].p.y = j;
     ants[an].state = 0; ants[an].resting = 0;
     ants[an].d = DirE; ants[an].has_food = 0;
     an++;
     break;
    default: if ((c >= '0') && (c <= '9'))
     {food[i][j] = c - 48; field[i][j] = Empty;}
     else assert (0); break;
   }
   assert (c != EOF);
  }
 fclose (f1);
}

void get_program (colour c)
{
 FILE * f1 = fopen (fn[c + 2], "rt");
 char str[MaxBufLen], cur[MaxBufLen];
 int n, k;
 _fmemset (instr[c], 0, sizeof (instr[c]));
 _fmemset (state1[c], 0, sizeof (state1[c]));
 _fmemset (state2[c], 0, sizeof (state2[c]));
 _fmemset (arg[c], 0, sizeof (arg[c]));
 for (n = 0; readstr (f1, str) != EOF; n++)
 {
  k = 0;
  readterm (cur, k, str);
//  strlwr (cur);
  if (!strcmp (cur, "Sense"))
  {
   instr[c][n] = Sense;
   readterm (cur, k, str);
   if (!strcmp (cur, "Here")) arg[c][n] = Here << 8;
   else if (!strcmp (cur, "Ahead")) arg[c][n] = Ahead << 8;
   else if (!strcmp (cur, "LeftAhead")) arg[c][n] = LeftAhead << 8;
   else if (!strcmp (cur, "RightAhead")) arg[c][n] = RightAhead << 8;
   else assert (0);
   readterm (cur, k, str);
   state1[c][n] = readnum (cur);
   readterm (cur, k, str);
   state2[c][n] = readnum (cur);
   readterm (cur, k, str);
   if (!strcmp (cur, "Friend")) arg[c][n] += Friend;
   else if (!strcmp (cur, "Foe")) arg[c][n] += Foe;
   else if (!strcmp (cur, "FriendWithFood")) arg[c][n] += FriendWithFood;
   else if (!strcmp (cur, "FoeWithFood")) arg[c][n] += FoeWithFood;
   else if (!strcmp (cur, "Food")) arg[c][n] += Food;
   else if (!strcmp (cur, "Rock")) arg[c][n] += Rocky; //!!!
   else if (!strcmp (cur, "Marker"))
   {
    readterm (cur, k, str);
    arg[c][n] += readnum (cur);
   }
   else if (!strcmp (cur, "FoeMarker")) arg[c][n] += FoeMarker;
   else if (!strcmp (cur, "Home")) arg[c][n] += Home;
   else if (!strcmp (cur, "FoeHome")) arg[c][n] += FoeHome;
   else assert (0);
   while ((str[k] > 0) && (str[k] <= 0x20)) k++;
   assert ((str[k] == 0) || (str[k] == ';'));
  }
  else if (!strcmp (cur, "Mark"))
  {
   instr[c][n] = Mark;
   readterm (cur, k, str);
   arg[c][n] += readnum (cur);
   readterm (cur, k, str);
   state1[c][n] += readnum (cur);
  }
  else if (!strcmp (cur, "Unmark"))
  {
   instr[c][n] = Unmark;
   readterm (cur, k, str);
   arg[c][n] += readnum (cur);
   readterm (cur, k, str);
   state1[c][n] += readnum (cur);
  }
  else if (!strcmp (cur, "PickUp"))
  {
   instr[c][n] = PickUp;
   readterm (cur, k, str);
   state1[c][n] += readnum (cur);
   readterm (cur, k, str);
   state2[c][n] += readnum (cur);
  }
  else if (!strcmp (cur, "Drop"))
  {
   instr[c][n] = Drop;
   readterm (cur, k, str);
   state1[c][n] += readnum (cur);
  }
  else if (!strcmp (cur, "Turn"))
  {
   instr[c][n] = Turn;
   readterm (cur, k, str);
   if (!strcmp (cur, "Left")) arg[c][n] = Left;
   else if (!strcmp (cur, "Right")) arg[c][n] = Right;
   else assert (0);
   readterm (cur, k, str);
   state1[c][n] += readnum (cur);
  }
  else if (!strcmp (cur, "Move"))
  {
   instr[c][n] = Move;
   readterm (cur, k, str);
   state1[c][n] += readnum (cur);
   readterm (cur, k, str);
   state2[c][n] += readnum (cur);
  }
  else if (!strcmp (cur, "Flip"))
  {
   instr[c][n] = Flip;
   readterm (cur, k, str);
   arg[c][n] += readnum (cur);
   readterm (cur, k, str);
   state1[c][n] += readnum (cur);
   readterm (cur, k, str);
   state2[c][n] += readnum (cur);
  }
  else assert (0);
 }
 fclose (f1);
}

void clear ()
{
 int i, j;
 for (i = 0; i <= MaxX; i++)
  for (j = 0; j <= MaxY; j++)
   pset (i, j, 0);
}

void add_colour (int & r1, int & g1, int & b1, int r2, int g2, int b2)
{
 r1 += r2; if (r1 > 5) r1 = 5; if (r1 < 0) r1 = 0;
 g1 += g2; if (g1 > 5) g1 = 5; if (g1 < 0) g1 = 0;
 b1 += b2; if (b1 > 5) b1 = 5; if (b1 < 0) b1 = 0;
}

byte get_colour (byte r1, byte g1, byte b1)
{
 return (r1 * 36 + g1 * 6 + b1);
}

void put_cell (int i, int j)
{
 int x = (i * xcell) + (j & 1) * (xcell >> 1), y = j * ycell;
 int dx, dy, c;
 int r1, r2, g1, g2, b1, b2;
 pos p = {i, j};
 if (some_ant_is_at (p))
 {
  if (get_colour (get_ant_at (p)) == Red)
   c = get_colour (colours[0][0], colours[0][1], colours[0][2]);
  else c = get_colour (colours[1][0], colours[1][1], colours[1][2]);
 }
 else if (get_food_at (p) > 0) c = 216 + (get_food_at (p) % 40);
 else
 {
  switch (field[i][j])
  {
   case Empty:
    r1 = colours[4][0]; g1 = colours[4][1]; b1 = colours[4][2];
    for (dx = 0; dx < 2; dx++)
     for (dy = 0; dy < 6; dy++)
      if ((marks[dx][i][j] & (1 << dy)) > 0)
      {
       r2 = colours[6 + dx * 6 + dy][0];
       g2 = colours[6 + dx * 6 + dy][1];
       b2 = colours[6 + dx * 6 + dy][2];
       add_colour (r1, g1, b1, r2, g2, b2);
       break;
      }
    c = get_colour (r1, g1, b1);
    break;
   case Rock:
    c = get_colour (colours[5][0], colours[5][1], colours[5][2]); break;
   case RedAnthill:
    c = get_colour (colours[2][0], colours[2][1], colours[2][2]); break;
   case BlackAnthill:
    c = get_colour (colours[3][0], colours[3][1], colours[3][2]); break;
   default: assert (0);
  }
 }
 for (dy = 0; dy < ycell; dy++)
  for (dx = 0; dx < xcell; dx++)
   pset (x + dx, y + dy, c);
}

void draw ()
{
 byte i, j;
 for (j = 0; j < ysz; j++)
  for (i = 0; i < xsz; i++)
   put_cell (i, j);
}

void dump_position (FILE * f1, unsigned long l)
{
 int i, j, k, n;
 pos p;
 fprintf (f1, "\nAfter round %lu...\n", l);
 for (j = 0; j < ysz; j++)
  for (i = 0; i < xsz; i++)
  {
   fprintf (f1, "cell (%d, %d): ", i, j);
   if (field[i][j] == Rock) fprintf (f1, "rock\n");
   else
   {
    p.x = i; p.y = j;
    if (get_food_at (p) > 0) fprintf (f1, "%d food; ", get_food_at (p));
    if (field[i][j] == RedAnthill) fprintf (f1, "red hill; ");
    if (field[i][j] == BlackAnthill) fprintf (f1, "black hill; ");
    if (marks[Red][i][j] > 0)
    {
     fprintf (f1, "red marks: ");
     for (k = 0; k < 6; k++)
      if ((marks[Red][i][j] & (1 << k)) > 0)
       putc (k + 48, f1);
     putc (';', f1); putc (' ', f1);
    }
    if (marks[Black][i][j] > 0)
    {
     fprintf (f1, "black marks: ");
     for (k = 0; k < 6; k++)
      if ((marks[Black][i][j] & (1 << k)) > 0)
       putc (k + 48, f1);
     putc (';', f1); putc (' ', f1);
    }
    if (some_ant_is_at (p))
    {
     n = get_ant_at (p);
     fprintf (f1, "%s ant of id %d, dir %d, food %d, state %d, resting %d",
      (ants[n].c == Red) ? "red" : "black",
      n, ants[n].d, ants[n].has_food, ants[n].state, ants[n].resting);
    }
    fprintf (f1, "\n");
   }
  }
}

void write_result (FILE * f1)
{
 int i, j;
 int sr = 0, sb = 0;
 for (j = 0; j < ysz; j++)
  for (i = 0; i < xsz; i++)
   if (field[i][j] == RedAnthill) sr += food[i][j];
   else if (field[i][j] == BlackAnthill) sb += food[i][j];
 fprintf (f1, "Total red food:   %d.\n", sr);
 fprintf (f1, "Total black food: %d.\n", sb);
 fprintf (f1, "%s\n",
  (sr == sb) ? "Draw." : ((sr > sb) ? "Red wins!" : "Black wins!"));
}

int main (int argc, char * argv[])
{
 int i, j;
 FILE * f1;
 for (i = 1; (i < argc) && (i < 2); i++)
  for (j = 0; (fn[i - 1][j] = argv[i][j]) != 0; j++) {}
 get_config ();
 if (to_draw)
 {
  init_gr_mode (0x13);
  set_palette ((byte *) palette[0], 0, 256);
  clear ();
/*
// Testing the graphics and palette if needed:
  for (i = 0; i < MaxX; i++)
   for (j = 0; j < MaxY; j++)
    pset (i, j, i * j);
  delay (5000);
*/
 }
 set_randomseed (new_rnd);
 if (to_dump)
 {
  f1 = fopen (fn[4], "wt");
/*
// Testing the randomseed if needed:
  fprintf (f1, "Testing the random seed:\n");
  for (i = 0; i < 100; i++)
   fprintf (f1, "%d\n", randomint (16384));
*/
  fprintf (f1, "random seed: %lu\n", new_rnd);
 }
 for (i = 2; (i < argc) && (i <= MaxFN); i++)
  for (j = 0; (fn[i - 1][j] = argv[i][j]) != 0; j++) {}
 get_board ();
 get_program (Red);
 get_program (Black);
/*
// Testing the world drawing if needed:
 draw ();
 delay (5000);
*/
 if (to_draw)
 {
  draw ();
  while (!kbhit ()) {}
  while (kbhit ()) getch ();
 }
 unsigned long l;
 for (l = 0; l < num_turns; l++)
 {
  if (to_draw && (l % draw_step == 0))
  {
   draw ();
   delay (delay1);
//   for (i = 0; i < 1; i++)
//    for (j = 0; j < 1; j++)
//     pset (300 + i, 100 + j, l & 0xFF);
  }
  if (to_dump && (l % dump_step == 0))
  {
   dump_position (f1, l);
  }
  for (i = 0; i < an; i++)
   step (i);
  if (kbhit ()) {while (kbhit ()) getch (); break;}
 }
 if (to_dump)
 {
  dump_position (f1, l);
 }
 if (to_draw)
 {
  draw ();
  while (!kbhit ()) {}
  while (kbhit ()) getch ();
  clear ();
  init_gr_mode (0x03);
 }
 if (to_dump)
 {
  fclose (f1);
 }
 f1 = fopen (fn[5], "wt");
 write_result (f1);
 fclose (f1);
 return (0);
}
