#include <assert.h>
#include <conio.h>
#include <dos.h>
#include <mem.h>
#include <stdio.h>
#include <string.h>

typedef unsigned char byte;
typedef signed char schar;
typedef char bool;

const byte XMax = 100, YMax = 100, MaxAnts = 200;
const int MaxState = 10000, MaxL = 11, MaxBufLen = 256;
typedef byte dir;
typedef byte lr_type;
typedef byte sense_dir;
typedef byte colour;
typedef byte condition;
typedef int state;
typedef byte instruction;
struct pos {int x, y;};
struct ant {colour c; pos p; int state; int resting; dir d; bool has_food;};
enum Edir {DirE, DirSSE, DirSSW, DirW, DirNNW, DirNNE};
enum Ecolour {Red = 0, Black = 1};
enum Elr_type {Left = 5, Right = 1};
enum Esense_dir {Here = 0, Ahead = 1, LeftAhead = 2, RightAhead = 3};
enum Efield {Empty = 0, Rock = 1, RedAnthill = 2, BlackAnthill = 3};
enum Econdition {Friend = 6, Foe, FriendWithFood, FoeWithFood, Food, Rocky,
 FoeMarker, Home, FoeHome};
enum Einstruction {Sense, Mark, Unmark, PickUp, Drop, Turn, Move, Flip};

char huge names[MaxState][MaxL];
char huge st1name[MaxState][MaxL];
char huge st2name[MaxState][MaxL];
instruction far instr[MaxState];
state far state1[MaxState];
state far state2[MaxState];
int far arg[MaxState];
int make_log = 1;

const MaxC = 18, MaxFN = 3;
char fn[MaxFN][90] = {"program.prg", "program.ant", "trans.log"};
unsigned long got_count, used_count;

int readstr (FILE * f1, char * str)
{
 int c, n = 0;
 while (((c = getc (f1)) != 0x0A) && (c != 0x0D) && (c != EOF))
 {
  str[n++] = c;
  assert (n < MaxBufLen);
 }
 str[n] = 0;
 if (c == 0x0D) c = getc (f1); // to read 0x0A
 return (c);
}

void readterm (char * ans, int & pos, char * str)
{
 for (; (str[pos] > 0) && (str[pos] <= 0x20); pos++) {}
 assert (str[pos] > 0);
 for (int k = 0; (ans[k] = str[pos]) > 0x20; k++, pos++) {}
 ans[k] = 0;
 if (str[pos] == 0x20) pos++;
}

int readnum (char * str)
{
 for (int n = 0, i = 0; (str[i] >= '0') && (str[i] <= '9'); i++)
 {
  n = n * 10 + str[i] - 48;
 }
 assert (i > 0);
 assert (str[i] == 0);
 return (n);
}

void get_program ()
{
 FILE * f1, * f2;
 f1 = fopen (fn[0], "rb");
 if (make_log) f2 = fopen (fn[2], "wb");
 char str[MaxBufLen], cur[MaxBufLen];
 int n, m, k;
 _fmemset (instr, 0, sizeof (instr));
 _fmemset (state1, 0, sizeof (state1));
 _fmemset (state2, 0, sizeof (state2));
 _fmemset (arg, 0, sizeof (arg));
 for (n = 0; readstr (f1, str) != EOF; n++)
 {
  while ((str[0] == '[') || (str[0] == 0))
  {assert (readstr (f1, str) != EOF);} // comment block or empty line
  if (make_log) {fprintf (f2, "Line %d: %s\n", n, str); fflush (f2);}
  state1[n] = MaxState;
  state2[n] = MaxState;
  k = 0;
  readterm (cur, k, str);
  for (m = 0; cur[m]; m++) {}
  m--;
  if (cur[m] == ':')
  {
   assert (m < MaxL);
   names[n][m] = 0;
   for (m--; m >= 0; m--) {names[n][m] = cur[m];}
   readterm (cur, k, str);
  }
  strlwr (cur);
  if (!strcmp (cur, "sense"))
  {
   instr[n] = Sense;
   readterm (cur, k, str);
   strlwr (cur);
   if (!strcmp (cur, "here")) arg[n] = Here << 8;
   else if (!strcmp (cur, "ahead")) arg[n] = Ahead << 8;
   else if (!strcmp (cur, "leftahead")) arg[n] = LeftAhead << 8;
   else if (!strcmp (cur, "rightahead")) arg[n] = RightAhead << 8;
   else assert (0);
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   readterm (cur, k, str);
   for (m = 0; (st2name[n][m] = cur[m]) > 0; m++) {}
   readterm (cur, k, str);
   strlwr (cur);
   if (!strcmp (cur, "friend")) arg[n] += Friend;
   else if (!strcmp (cur, "foe")) arg[n] += Foe;
   else if (!strcmp (cur, "friendwithfood")) arg[n] += FriendWithFood;
   else if (!strcmp (cur, "foewithfood")) arg[n] += FoeWithFood;
   else if (!strcmp (cur, "food")) arg[n] += Food;
   else if (!strcmp (cur, "rock")) arg[n] += Rocky; //!!!
   else if (!strcmp (cur, "marker"))
   {
    readterm (cur, k, str);
    arg[n] += readnum (cur);
   }
   else if (!strcmp (cur, "foemarker")) arg[n] += FoeMarker;
   else if (!strcmp (cur, "home")) arg[n] += Home;
   else if (!strcmp (cur, "foehome")) arg[n] += FoeHome;
   else assert (0);
   while ((str[k] > 0) && (str[k] <= 0x20)) k++;
   assert ((str[k] == 0) || (str[k] == ';'));
  }
  else if (!strcmp (cur, "mark"))
  {
   instr[n] = Mark;
   readterm (cur, k, str);
   arg[n] += readnum (cur);
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   st2name[n][0] = 0;
  }
  else if (!strcmp (cur, "unmark"))
  {
   instr[n] = Unmark;
   readterm (cur, k, str);
   arg[n] += readnum (cur);
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   st2name[n][0] = 0;
  }
  else if (!strcmp (cur, "pickup"))
  {
   instr[n] = PickUp;
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   readterm (cur, k, str);
   for (m = 0; (st2name[n][m] = cur[m]) > 0; m++) {}
  }
  else if (!strcmp (cur, "drop"))
  {
   instr[n] = Drop;
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   st2name[n][0] = 0;
  }
  else if (!strcmp (cur, "turn"))
  {
   instr[n] = Turn;
   readterm (cur, k, str);
   strlwr (cur);
   if (!strcmp (cur, "left")) arg[n] = Left;
   else if (!strcmp (cur, "right")) arg[n] = Right;
   else assert (0);
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   st2name[n][0] = 0;
  }
  else if (!strcmp (cur, "move"))
  {
   instr[n] = Move;
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   readterm (cur, k, str);
   for (m = 0; (st2name[n][m] = cur[m]) > 0; m++) {}
  }
  else if (!strcmp (cur, "flip"))
  {
   instr[n] = Flip;
   readterm (cur, k, str);
   arg[n] += readnum (cur);
   readterm (cur, k, str);
   for (m = 0; (st1name[n][m] = cur[m]) > 0; m++) {}
   readterm (cur, k, str);
   for (m = 0; (st2name[n][m] = cur[m]) > 0; m++) {}
  }
  else assert (0);
 }
 got_count = n;
 fclose (f1);
 if (make_log) {fprintf (f2, "All OK.\n"); fflush (f2);}
 if (make_log) fclose (f2);
}

int build_state (char * cur, int n)
{
 int k;
 if (cur[0] == '+')
 {
  k = readnum (& (cur[1]));
  assert (n + k < used_count);
  return (n + k);
 }
 else if (cur[0] == '-')
 {
  k = readnum (& (cur[1]));
  assert (n - k >= 0);
  return (n - k);
 }
 else if ((cur[0] >= '0') && (cur[0] <= '9'))
 {
  k = readnum (cur);
  return (k);
 }
 else if (!strcmp (cur, "next") || !strcmp (cur, "."))
 {
  assert (n + 1 < used_count);
  return (n + 1);
 }
 else
 {
  int i;
  for (i = 0; i < got_count; i++)
  {
   if (!strcmp (cur, (char *) names[i])) break;
  }
  assert (i < got_count);
  return (i);
 }
}

void modify_program ()
{
 used_count = got_count;
 int n;
 for (n = 0; n < got_count; n++)
 {
  if (st1name[n][0])
  {
   state1[n] = build_state ((char *) st1name[n], n);
  }
  if (st2name[n][0])
  {
   state2[n] = build_state ((char *) st2name[n], n);
  }
 }
}

void put_program ()
{
 FILE * f1 = fopen (fn[1], "wb");
 int n;
 for (n = 0; n < used_count; n++)
 {
  switch (instr[n])
  {
   case Sense:
    fprintf (f1, "Sense ");
    switch (arg[n] >> 8)
    {
     case Here: fprintf (f1, "Here "); break;
     case Ahead: fprintf (f1, "Ahead "); break;
     case LeftAhead: fprintf (f1, "LeftAhead "); break;
     case RightAhead: fprintf (f1, "RightAhead "); break;
     default: assert (0);
    }
    fprintf (f1, "%d ", state1[n]);
    fprintf (f1, "%d ", state2[n]);
    switch (arg[n] & 0xFF)
    {
     case Friend: fprintf (f1, "Friend\n"); break;
     case Foe: fprintf (f1, "Foe\n"); break;
     case FriendWithFood: fprintf (f1, "FriendWithFood\n"); break;
     case FoeWithFood: fprintf (f1, "FoeWithFood\n"); break;
     case Food: fprintf (f1, "Food\n"); break;
     case Rocky: fprintf (f1, "Rock\n"); break;
     case FoeMarker: fprintf (f1, "FoeMarker\n"); break;
     case Home: fprintf (f1, "Home\n"); break;
     case FoeHome: fprintf (f1, "FoeHome\n"); break;
     default: if ((arg[n] & 0xFF) <= 5)
      fprintf (f1, "Marker %d\n", arg[n] & 0xFF);
      else assert (0);
    }
    break;
   case Mark:
    fprintf (f1, "Mark %d %d\n", arg[n], state1[n]);
    break;
   case Unmark:
    fprintf (f1, "Unmark %d %d\n", arg[n], state1[n]);
    break;
   case PickUp:
    fprintf (f1, "PickUp %d %d\n", state1[n], state2[n]);
    break;
   case Drop:
    fprintf (f1, "Drop %d\n", state1[n]);
    break;
   case Turn:
    fprintf (f1, "Turn ");
    switch (arg[n])
    {
     case Left: fprintf (f1, "Left "); break;
     case Right: fprintf (f1, "Right "); break;
     default: assert (0);
    }
    fprintf (f1, "%d\n", state1[n]);
    break;
   case Move:
    fprintf (f1, "Move %d %d\n", state1[n], state2[n]);
    break;
   case Flip:
    fprintf (f1, "Flip %d %d %d\n", arg[n], state1[n], state2[n]);
    break;
   default: assert (0);
  }
 }
 fclose (f1);
}

int main (int argc, char * argv[])
{
 int i, j;
 for (i = 1; (i < argc) && (i <= MaxFN); i++)
  for (j = 0; (fn[i - 1][j] = argv[i][j]) != 0; j++) {}
 printf ("Reading program from file `%s'...", fn[0]);
 get_program ();
 printf (" Done.\n");
 printf ("Transforming program into automaton...");
 modify_program ();
 printf (" Done.\n");
 printf ("Writing automaton to file `%s'...", fn[1]);
 put_program ();
 printf (" Done.\n");
 return (0);
}
